@@ -20,91 +20,43 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
-=head1 NAME
-
-mt-aws-glacier - Perl Multithreaded Multipart sync to Amazon Glacier
-
-=head1 SYNOPSIS
-
-More info in README.md or L<https://github.com/vsespb/mt-aws-glacier> or L<http://mt-aws.com/>
-
-=cut
-
-
 package App::MtAws;
 
 use strict;
 use warnings;
 use utf8;
-use 5.008008; # minumum perl version is 5.8.8
 
-our $VERSION = '0.981_01';
-our $VERSION_MATURITY = "beta";
+our $VERSION = "0.957beta";
 
 use constant ONE_MB => 1024*1024;
 
 use App::MtAws::ParentWorker;
 use App::MtAws::ChildWorker;
 use App::MtAws::JobProxy;
-use App::MtAws::Job::FileCreate;
-use App::MtAws::Job::FileListDelete;
-use App::MtAws::Job::RetrievalFetch;
+use App::MtAws::FileCreateJob;
+use App::MtAws::FileListDeleteJob;
+use App::MtAws::FileListRetrievalJob;
+use App::MtAws::RetrievalFetchJob;
 use App::MtAws::JobListProxy;
-use App::MtAws::Job::RetrieveInventory;
+use App::MtAws::RetrieveInventoryJob;
+use App::MtAws::InventoryFetchJob;
 use File::Find ;
 use File::Spec;
 use App::MtAws::Journal;
 use App::MtAws::ConfigDefinition;
 use App::MtAws::ForkEngine qw/with_forks fork_engine/;
 use Carp;
+use File::stat;
 use IO::Handle;
-use App::MtAws::Job::CreateVault;
-use App::MtAws::Job::DeleteVault;
+use App::MtAws::CreateVaultJob;
+use App::MtAws::DeleteVaultJob;
 use App::MtAws::Utils;
 use App::MtAws::Exceptions;
 use PerlIO::encoding;
 
-sub check_module_versions
-{
-	for (keys %INC) {
-		if (my ($mod) = /^App\/MtAws\/(.*)\.pmc?$/) {
-			$mod =~ s!/!::!g;
-			my $module = "App::MtAws::$mod";
-			my $got = $module->VERSION;
-			$got = 'undef' unless defined $got;
-			die "FATAL: wrong version of $module, expected $VERSION, found $got" unless $got eq $VERSION;
-		}
-	};
-}
-
-sub print_system_modules_version
-{
-	for my $module (sort keys %INC) {
-		if ($module !~ /^App\/MtAws/ && $module =~ /\.pmc?/) {
-			my $name = $module;
-			$name =~ s[/][::]g;
-			$name =~ s[\.pmc?$][];
-			my $ver = $name->VERSION;
-			$ver = 'undef' unless defined $ver;
-			print "$name\t$ver\t$INC{$module}\n";
-		}
-	}
-}
-
-sub load_all_dynamic_modules
-{
-	# we load here all dynamically loaded modules, to check that installation is correct.
-	require App::MtAws::Command::Sync;
-	require App::MtAws::Command::Retrieve;
-	require App::MtAws::Command::CheckLocalHash;
-	require App::MtAws::Command::DownloadInventory;
-	check_module_versions;
-}
 
 sub main
 {
-	check_module_versions();
 	unless (defined eval {process(); 1;}) {
 		dump_error(q{});
 		exit(1);
@@ -116,22 +68,22 @@ sub main
 sub process
 {
 	$|=1;
-	STDERR->autoflush(1);
-	print "MT-AWS-Glacier, Copyright 2012-2013 Victor Efimov http://mt-aws.com/ Version $VERSION$VERSION_MATURITY\n\n";
-
+	STDERR->autoflush(1); 
+	print "MT-AWS-Glacier, Copyright 2012-2013 Victor Efimov http://mt-aws.com/ Version $VERSION\n\n";
+	
 	my ($P) = @_;
 	my ($src, $vault, $journal);
 	my $maxchildren = 4;
 	my $config = {};
 	my $config_filename;
-
-
+	
+	
 	my $res = App::MtAws::ConfigDefinition::get_config()->parse_options(@ARGV);
 	my ($action, $options) = ($res->{command}, $res->{options});
 	if ($res->{warnings}) {
 		while (@{$res->{warnings}}) {
 			my ($warning, $warning_text) = (shift @{$res->{warnings}}, shift @{$res->{warning_texts}});
-			print STDERR "WARNING: $warning_text\n";
+			print STDERR "WARNING: $warning_text\n" unless $warning->{format} =~ /^(deprecated_option|deprecated_command|option_deprecated_for_command)$/; # TODO: temporary disable warning
 		}
 	}
 	if ($res->{error_texts}) {
@@ -140,81 +92,103 @@ sub process
 		}
 		die exception cmd_error => 'Error in command line/config'
 	}
-	if ($action ne 'help' && $action ne 'version') {
+	if ($action ne 'help') {
 		$PerlIO::encoding::fallback = Encode::FB_QUIET;
 		binmode STDERR, ":encoding($options->{'terminal-encoding'})";
 		binmode STDOUT, ":encoding($options->{'terminal-encoding'})";
 	}
-
-	my %journal_opts = ( journal_encoding => $options->{'journal-encoding'} );
-
+	
+	my %journal_opts = ( journal_encoding => $options->{'journal-encoding'}, filenames_encoding => $options->{'filenames-encoding'} );
+	
 	if ($action eq 'sync') {
 		die "Not a directory $options->{dir}" unless -d binaryfilename $options->{dir};
-
+		
+		my $partsize = delete $options->{partsize};
+		
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal}, root_dir => $options->{dir},
-			filter => $options->{filters}{parsed}, leaf_optimization => $options->{'leaf-optimization'}, follow => $options->{'follow'});
-
-		require App::MtAws::Command::Sync;
-		check_module_versions;
-		App::MtAws::Command::Sync::run($options, $j);
-
+			filter => $options->{filters}{parsed}, leaf_optimization => $options->{'leaf-optimization'});
+		
+		with_forks !$options->{'dry-run'}, $options, sub {
+			$j->read_journal(should_exist => 0);
+			$j->read_new_files($options->{'max-number-of-files'});
+			
+			if ($options->{'dry-run'}) {
+				for (@{ $j->{newfiles_a} }) {
+					my ($absfilename, $relfilename) = ($j->absfilename($_->{relfilename}), $_->{relfilename});
+					print "Will UPLOAD $absfilename\n";
+				}
+			} else {
+				$j->open_for_write();
+				
+				my @joblist;
+				for (@{ $j->{newfiles_a} }) {
+					my ($absfilename, $relfilename) = ($j->absfilename($_->{relfilename}), $_->{relfilename});
+					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::FileCreateJob->new(filename => $absfilename, relfilename => $relfilename, partsize => ONE_MB*$partsize));
+					push @joblist, $ft;
+				}
+				if (scalar @joblist) {
+					my $lt = App::MtAws::JobListProxy->new(jobs => \@joblist);
+					my ($R) = fork_engine->{parent_worker}->process_task($lt, $j);
+					die unless $R;
+				}
+				$j->close_for_write();
+			}
+		}
 	} elsif ($action eq 'upload-file') {
-
+		
 		defined(my $relfilename = $options->{relfilename})||confess;
 		my $partsize = delete $options->{partsize};
-
+		
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal});
-
+		
 		with_forks 1, $options, sub {
-
+			
 			$j->read_journal(should_exist => 0);
-
-			## no Test::Tabs
+			
 			die <<"END"
 File with same name alredy exists in Journal.
 In the current version of mtglacier you are disallowed to store multiple versions of same file.
 Multiversion will be implemented in the future versions.
 END
 				if (defined $j->{journal_h}->{$relfilename});
-			## use Test::Tabs
-
+			
 			if ($options->{'data-type'} ne 'filename') {
 				binmode STDIN;
 				check_stdin_not_empty(); # after we fork, but before we touch Journal for write and create Amazon Glacier upload id
 			}
-
+			
 			$j->open_for_write();
-
+			
 			my $ft = ($options->{'data-type'} eq 'filename') ?
-				App::MtAws::JobProxy->new(job => App::MtAws::Job::FileCreate->new(
+				App::MtAws::JobProxy->new(job => App::MtAws::FileCreateJob->new(
 					filename => $options->{filename},
 					relfilename => $relfilename,
 					partsize => ONE_MB*$partsize)) :
-				App::MtAws::JobProxy->new(job => App::MtAws::Job::FileCreate->new(
+				App::MtAws::JobProxy->new(job => App::MtAws::FileCreateJob->new(
 					stdin => 1,
 					relfilename => $relfilename,
 					partsize => ONE_MB*$partsize));
-
+			
 			my ($R) = fork_engine->{parent_worker}->process_task($ft, $j);
 			die unless $R;
 			$j->close_for_write();
 		}
 	} elsif ($action eq 'purge-vault') {
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal}, filter => $options->{filters}{parsed});
-
+		
 		with_forks !$options->{'dry-run'}, $options, sub {
 			$j->read_journal(should_exist => 1);
-
-			my $archives = $j->{archive_h};
-			if (scalar keys %$archives) {
+			
+			my $files = $j->{journal_h};
+			if (scalar keys %$files) {
 				if ($options->{'dry-run'}) {
-					for (keys %$archives) {
-						print "Will DELETE archive $_ (filename $archives->{$_}{relfilename})\n"
-					}
+						for (keys %$files) {
+							print "Will DELETE archive $files->{$_}{archive_id} (filename $_)\n"
+						}
 				} else {
 					$j->open_for_write();
-					my @filelist = map { {archive_id => $_, relfilename =>$archives->{$_}->{relfilename} } } keys %{$archives};
-					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::Job::FileListDelete->new(archives => \@filelist ));
+					my @filelist = map { {archive_id => $files->{$_}->{archive_id}, relfilename =>$_ } } keys %{$files};
+					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::FileListDeleteJob->new(archives => \@filelist ));
 					my ($R) = fork_engine->{parent_worker}->process_task($ft, $j);
 					die unless $R;
 					$j->close_for_write();
@@ -226,36 +200,25 @@ END
 	} elsif ($action eq 'restore') {
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal}, root_dir => $options->{dir}, filter => $options->{filters}{parsed}, use_active_retrievals => 1);
 		confess unless $options->{'max-number-of-files'};
-
-
-		require App::MtAws::Command::Retrieve;
-		check_module_versions;
-		App::MtAws::Command::Retrieve::run($options, $j);
+				
+		require App::MtAws::RetrieveCommand;
+		App::MtAws::RetrieveCommand::run($options, $j);
 	} elsif ($action eq 'restore-completed') {
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal}, root_dir => $options->{dir}, filter => $options->{filters}{parsed});
-
+		
 		with_forks !$options->{'dry-run'}, $options, sub {
 			$j->read_journal(should_exist => 1);
-
+			
 			my $files = $j->{journal_h};
 			# TODO: refactor
-			my %filelist = map { $_->{archive_id} => $_ }
-				grep { !-f binaryfilename $_->{filename} }
-				map {
-					my $entry = $j->latest($_);
-					{
-						archive_id => $entry->{archive_id}, mtime => $entry->{mtime}, size => $entry->{size},
-						treehash => $entry->{treehash}, relfilename =>$_, filename=> $j->absfilename($_)
-					}
-				}
-				keys %{$files};
+			my %filelist =	map { $_->{archive_id} => $_ } grep { ! binaryfilename -f $_->{filename} } map { {archive_id => $files->{$_}->{archive_id}, mtime => $files->{$_}{mtime}, relfilename =>$_, filename=> $j->absfilename($_) } } keys %{$files};
 			if (keys %filelist) {
 				if ($options->{'dry-run'}) {
-					for (keys %filelist) {
-						print "Will DOWNLOAD (if available) archive $_->{archive_id} (filename $_->{relfilename})\n" for ($j->latest($_));
+					for (values %filelist) {
+						print "Will DOWNLOAD (if available) archive $_->{archive_id} (filename $_->{relfilename})\n"
 					}
 				} else {
-					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::Job::RetrievalFetch->new(file_downloads => $options->{file_downloads}, archives => \%filelist ));
+					my $ft = App::MtAws::JobProxy->new(job => App::MtAws::RetrievalFetchJob->new(archives => \%filelist ));
 					my ($R) = fork_engine->{parent_worker}->process_task($ft, $j);
 					die unless $R;
 				}
@@ -265,38 +228,77 @@ END
 		}
 	} elsif ($action eq 'check-local-hash') {
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{journal}, root_dir => $options->{dir}, filter => $options->{filters}{parsed});
-		require App::MtAws::Command::CheckLocalHash;
-		check_module_versions;
-		App::MtAws::Command::CheckLocalHash::run($options, $j);
+		require App::MtAws::CheckLocalHashCommand;
+		App::MtAws::CheckLocalHashCommand::run($options, $j);
 	} elsif ($action eq 'retrieve-inventory') {
 		$options->{concurrency} = 1; # TODO implement this in ConfigEngine
-
+				
 		with_forks 1, $options, sub {
-			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::Job::RetrieveInventory->new());
+			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::RetrieveInventoryJob->new());
 			my ($R) = fork_engine->{parent_worker}->process_task($ft, undef);
 		}
 	} elsif ($action eq 'download-inventory') {
 		$options->{concurrency} = 1; # TODO implement this in ConfigEngine
 		my $j = App::MtAws::Journal->new(%journal_opts, journal_file => $options->{'new-journal'});
-		require App::MtAws::Command::DownloadInventory;
-		check_module_versions;
-		App::MtAws::Command::DownloadInventory::run($options, $j);
+				
+		with_forks 1, $options, sub {
+			
+			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::InventoryFetchJob->new());
+			my ($R, $attachmentref) = fork_engine->{parent_worker}->process_task($ft, undef);
+			# here we can have response from both JobList or Inventory output..
+			# JobList looks like 'response' => '{"JobList":[],"Marker":null}'
+			# Inventory retriebal has key 'ArchiveList'
+			# TODO: implement it more clear way on level of Job/Tasks object
+			
+			croak if -s binaryfilename $options->{'new-journal'}; # TODO: fix race condition between this and opening file
+			
+			if ($R && $attachmentref) {
+				$j->open_for_write();
+	
+				my $data = JSON::XS->new->allow_nonref->utf8->decode($$attachmentref);
+				my $now = time();
+				
+				for my $item (@{$data->{'ArchiveList'}}) {
+					
+					my ($relfilename, $mtime) = App::MtAws::MetaData::meta_decode($item->{ArchiveDescription});
+					$relfilename = $item->{ArchiveId} unless defined $relfilename;
+					$mtime = $now unless defined $mtime;
+					
+					my $creation_time = App::MtAws::MetaData::_parse_iso8601($item->{CreationDate}); # TODO: move code out
+					#time archive_id size mtime treehash relfilename
+					$j->add_entry({
+						type => 'CREATED',
+						relfilename => $relfilename,
+						time => $creation_time,
+						archive_id => $item->{ArchiveId},
+						size => $item->{Size},
+						mtime => $mtime,
+						treehash => $item->{SHA256TreeHash},
+					});
+				}
+				$j->close_for_write();
+			}
+		}
 	} elsif ($action eq 'create-vault') {
 		$options->{concurrency} = 1;
-
+				
 		with_forks 1, $options, sub {
-			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::Job::CreateVault->new(name => $options->{'vault-name'}));
+			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::CreateVaultJob->new(name => $options->{'vault-name'}));
 			my ($R) = fork_engine->{parent_worker}->process_task($ft, undef);
 		}
 	} elsif ($action eq 'delete-vault') {
 		$options->{concurrency} = 1;
-
+				
 		with_forks 1, $options, sub {
-			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::Job::DeleteVault->new(name => $options->{'vault-name'}));
+			my $ft = App::MtAws::JobProxy->new(job => App::MtAws::DeleteVaultJob->new(name => $options->{'vault-name'}));
 			my ($R) = fork_engine->{parent_worker}->process_task($ft, undef);
 		}
 	} elsif ($action eq 'help') {
-## no Test::Tabs
+		
+		# we load here all dynamically loaded modules, to check that installation is correct.
+		require App::MtAws::CheckLocalHashCommand;
+		require App::MtAws::RetrieveCommand;
+		
 		print <<"END";
 Usage: mtglacier.pl COMMAND [POSITIONAL ARGUMENTS] [OPTION]...
 
@@ -311,30 +313,23 @@ Common options:
 	--partsize - Glacier multipart upload part size
 	--filter --include --exclude - File filtering
 	--dry-run - Don't do anything
-	--token - to be used with STS/IAM
-	--timeout - socket timeout
 Commands:
 	sync
-	  --new --replace-modified --delete-removed - Sync modes
-	  --leaf-optimization - Don't use directory hardlinks count when traverse.
-	  --follow - Follow symbolic links
-	  --detect treehash|mtime|mtime-or-treehash|mtime-and-treehash|always-positive|size-only
+		--leaf-optimization - Don't use directory hardlinks count when traverse.
 	purge-vault
 	restore
 	restore-completed
-	  --segment-size - Size for multi-segment download, in megabytes
 	check-local-hash
 	retrieve-inventory
 	download-inventory
-	  --new-journal - Write inventory as new journal
+		--new-journal - Write inventory as new journal
 	create-vault VAULT-NAME
 	delete-vault VAULT-NAME
 	upload-file
-	  --filename - File to upload
-	  --set-rel-filename - Relative filename to use in Journal (if dir not specified)
-	  --stdin - Upload from STDIN
-	  --check-max-file-size - Specify to ensure there will be less than 10 000 parts
-	version - prints debug information about software installed
+		--filename - File to upload
+		--set-rel-filename - Relative filename to use in Journal (if dir not specified)
+		--stdin - Upload from STDIN
+		--check-max-file-size - Specify to ensure there will be less than 10 000 parts
 Config format (text file):
 	key=YOURKEY
 	secret=YOURSECRET
@@ -342,15 +337,9 @@ Config format (text file):
 	region=us-east-1
 	# protocol=http (default) or https
 	protocol=http
+
 END
-
-## use Test::Tabs
-
-	} elsif ($action eq 'version') {
-		load_all_dynamic_modules();
-		print "mt-aws-glacier version: $VERSION $VERSION_MATURITY\n";
-		print "Perl Version: $]\n";
-		print_system_modules_version();
+	
 	} else {
 		die "Wrong usage";
 	}
