@@ -20,10 +20,9 @@
 
 package App::MtAws::Exceptions;
 
-our $VERSION = '0.981_01';
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use utf8;
 use Carp;
 
@@ -78,18 +77,16 @@ sub exception_message
 	my $rep = sub {
 		my ($match) = @_;
 		if (my ($format, $name) = $match =~ /^([\w]+)\s+([\w]+)$/) {
-			my $value = $data{$name};
-			if (defined($value)) {
-				if (lc $format eq lc 'string') {
-					qq{"$value"};
-				} else {
-					sprintf("%$format", $value);
-				}
+			if (lc $format eq lc 'string') {
+				defined(my $value = $data{$name})||confess;
+				qq{"$value"};
 			} else {
-				':NULL:'
+				defined(my $value = $data{$name})||confess;
+				sprintf("%$format", $value);
 			}
 		} else {
-			defined($data{$match}) ? $data{$match} : ':NULL:';
+			defined(my $value = $data{$match})||confess $spec;
+			$value;
 		}
 	};
 	
@@ -106,7 +103,7 @@ sub dump_error
 	if (is_exception('cmd_error')) {
 		# no additional output
 	} elsif (is_exception) {
-		print STDERR "ERROR$where: ".exception_message($@)."\n";
+		print STDERR "ERROR$where: $@->{message}\n";
 	} else {
 		print STDERR "UNEXPECTED ERROR$where: $@\n";
 	}
