@@ -20,8 +20,6 @@
 
 package App::MtAws::ForkEngine;
 
-our $VERSION = '0.981_01';
-
 use strict;
 use warnings;
 use utf8;
@@ -72,34 +70,14 @@ sub with_forks($$&)
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	$self->{options}||die;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    $self->{options}||die;
 	$self->{children} = {};
 #    $self->{disp_select}||die;
 #    @{$self->{freeworkers}} = keys %{$self->{children}};
-	bless $self, $class;
-	return $self;
-}
-
-sub run_children
-{
-	my ($self, $child_fromchild, $child_tochild) = @_;
-	my $C = App::MtAws::ChildWorker->new(options => $self->{options}, fromchild => $child_fromchild, tochild => $child_tochild);
-	dump_error("child $$") unless (defined eval {$C->process(); 1;});
-}
-
-sub run_parent
-{
-	my ($self, $disp_select) = @_;
-	return $self->{parent_worker} = App::MtAws::ParentWorker->new(children => $self->{children}, disp_select => $disp_select, options=>$self->{options});
-}
-
-sub parent_exit_on_signal
-{
-	my ($self, $sig, $children) = @_;
-	print STDERR "\nEXIT on SIG$sig\n";
-	exit(1);
+    bless $self, $class;
+    return $self;
 }
 
 sub start_children
@@ -107,47 +85,38 @@ sub start_children
 	my ($self) = @_;
 	# parent's data
 	my $disp_select = IO::Select->new();
-	$self->{parent_pid} = $$;
+	my $parent_pid = $$;
 	# child/parent code
 	for my $n (1..$self->{options}->{concurrency}) {
 		my ($ischild, $child_fromchild, $child_tochild) = $self->create_child($disp_select);
 		if ($ischild) {
 			# child code
-			my $first_time = 1;
-			my @signals = qw/INT TERM USR2 HUP/;
-			for my $sig (@signals) {
-				$SIG{$sig} = sub {
-					if ($first_time) {
-						$first_time = 0;
-						exit(1); # we need exit, it will call all destructors which will destroy tempfiles
-					}
-				};
+			my $C = App::MtAws::ChildWorker->new(options => $self->{options}, fromchild => $child_fromchild, tochild => $child_tochild);
+
+			unless (defined eval {$C->process(); 1;}) {
+				dump_error(q{child});
+				exit(1);
 			}
-			$self->run_children($child_fromchild, $child_tochild);
+			
+			kill(POSIX::SIGUSR1, $parent_pid);
 			exit(1);
 		}
 	}
-
+	
 	my $first_time = 1;
-	for my $sig (qw/INT TERM CHLD USR1 HUP/) {
+	for my $sig (qw/INT TERM CHLD USR1/) {
 		$SIG{$sig} = sub {
 			local ($!,$^E,$@);
-			if ($sig eq 'CHLD') {
-				my $pid = waitpid(-1, WNOHANG);
-				# make sure we caugth signal from our children, not from external command executionin 3rd party module
-				# easy to test by adding `whoami` to parent after-fork-code
-				return unless $pid > 0 and defined $self->{children}{$pid};
-			}
 			if ($first_time) {
 				$first_time = 0;
 				kill (POSIX::SIGUSR2, keys %{$self->{children}});
-				while( wait() != -1 ){};
-				$self->parent_exit_on_signal($sig, $self->{children});
+				print STDERR "EXIT on SIG$sig\n";
+				exit(1);
 			}
 		};
 	}
-
-	return $self->run_parent($disp_select);
+	
+	return $self->{parent_worker} = App::MtAws::ParentWorker->new(children => $self->{children}, disp_select => $disp_select, options=>$self->{options});
 }
 
 #
@@ -155,63 +124,63 @@ sub start_children
 #
 sub create_child
 {
-	my ($self, $disp_select) = @_;
+  my ($self, $disp_select) = @_;
+  
+  my $fromchild = new IO::Pipe;
+  #log("created fromchild pipe $!", 10) if level(10);
+  my $tochild = new IO::Pipe;
+  #log("created tochild pipe $!", 10) if level(10);
+  my $pid;
+  my $parent_pid = $$;
+  
+  if($pid = fork()) { # Parent
+   $|=1;
+   STDERR->autoflush(1);
+   $fromchild->reader();
+   $fromchild->autoflush(1);
+   $fromchild->blocking(1);
+   binmode $fromchild;
+   
+   $tochild->writer();
+   $tochild->autoflush(1);
+   $tochild->blocking(1);
+   binmode $tochild;
+    
+   $disp_select->add($fromchild);
+   $self->{children}->{$pid} = { pid => $pid, fromchild => $fromchild, tochild => $tochild };
+   
+   print "PID $pid Started worker\n";
+   return (0, undef, undef);
+  } elsif (defined ($pid)) { # Child
+   $|=1;
+   STDERR->autoflush(1);
+   $fromchild->writer();
+   $fromchild->autoflush(1);
+   $fromchild->blocking(1);
+   binmode $fromchild; 
 
-	my $fromchild = new IO::Pipe;
-	#log("created fromchild pipe $!", 10) if level(10);
-	my $tochild = new IO::Pipe;
-	#log("created tochild pipe $!", 10) if level(10);
-	my $pid;
-	my $parent_pid = $$;
+   $tochild->reader();
+   $tochild->autoflush(1);
+   $tochild->blocking(1);
+   binmode $tochild;
 
-	if($pid = fork()) { # Parent
-		$|=1;
-		STDERR->autoflush(1);
-		$fromchild->reader();
-		$fromchild->autoflush(1);
-		$fromchild->blocking(1);
-		binmode $fromchild;
-
-		$tochild->writer();
-		$tochild->autoflush(1);
-		$tochild->blocking(1);
-		binmode $tochild;
-
-		$disp_select->add($fromchild);
-		$self->{children}->{$pid} = { pid => $pid, fromchild => $fromchild, tochild => $tochild };
-
-		print "PID $pid Started worker\n";
-		return (0, undef, undef);
-	} elsif (defined ($pid)) { # Child
-		$|=1;
-		STDERR->autoflush(1);
-		$fromchild->writer();
-		$fromchild->autoflush(1);
-		$fromchild->blocking(1);
-		binmode $fromchild;
-
-		$tochild->reader();
-		$tochild->autoflush(1);
-		$tochild->blocking(1);
-		binmode $tochild;
-
-		undef $disp_select; # we discard tonns of unneeded pipes !
-		undef $self->{children};
-
-		return (1, $fromchild, $tochild);
-	} else {
-		die "Cannot fork()";
-	}
+   
+   undef $disp_select; # we discard tonns of unneeded pipes !
+   undef $self->{children};
+   
+  
+   return (1, $fromchild, $tochild);
+  } else {
+  	die "Cannot fork()";
+  }
 }
 
 
 sub terminate_children
 {
 	my ($self) = @_;
-	$SIG{CHLD} = 'DEFAULT'; # don't set SIGCHLD to IGNORE, prevents wait() from working under 5.12.2,3 undef OpenBSD
-	$SIG{INT} = $SIG{USR2}='IGNORE';
+	$SIG{INT} = $SIG{TERM} = $SIG{CHLD} = $SIG{USR2}='IGNORE';
 	kill (POSIX::SIGUSR2, keys %{$self->{children}}); # TODO: we terminate all children with SIGUSR2 even on normal exit
-	$SIG{TERM} = 'DEFAULT';
-	while( wait() != -1 ){};
+	while(wait() != -1) { print STDERR "wait\n";};
 }
 1;
