@@ -20,46 +20,45 @@
 
 package App::MtAws::GlacierRequest;
 
-our $VERSION = '0.981_01';
-
 use strict;
 use warnings;
 use utf8;
 use POSIX;
-use LWP 5.803;
 use LWP::UserAgent;
-use HTTP::Request;
+use HTTP::Request::Common;
+use App::MtAws::TreeHash;
 use Digest::SHA qw/hmac_sha256 hmac_sha256_hex sha256_hex sha256/;
 use App::MtAws::MetaData;
 use App::MtAws::Utils;
 use App::MtAws::Exceptions;
-use App::MtAws::HttpSegmentWriter;
 use Carp;
 
 
 
+
 sub new
 {
 	my ($class, $options) = @_;
 	my $self = {};
 	bless $self, $class;
-
-	defined($self->{$_} = $options->{$_})||confess $_ for (qw/region key secret protocol timeout/);
+	
+	defined($self->{$_} = $options->{$_})||confess $_ for (qw/region key secret protocol/);
 	defined($options->{$_}) and $self->{$_} = $options->{$_} for (qw/vault token/); # TODO: validate vault later
-
+	
+	
 	confess unless $self->{protocol} =~ /^https?$/; # we check external data here, even if it's verified in the beginning, especially if it's used to construct URL
 	$self->{service} ||= 'glacier';
 	$self->{account_id} = '-';
 	$self->{host} = "$self->{service}.$self->{region}.amazonaws.com";
 
 	$self->{headers} = [];
-
+   
 	$self->add_header('Host', $self->{host});
 	$self->add_header('x-amz-glacier-version', '2012-06-01') if $self->{service} eq 'glacier';
 	$self->add_header('x-amz-security-token', $self->{token}) if defined $self->{token};
-
-	return $self;
-}
+	
+	return $self;                                                                                                                                                                                                                                                                     
+}                      
 
 sub add_header
 {
@@ -70,16 +69,16 @@ sub add_header
 sub create_multipart_upload
 {
 	my ($self, $partsize, $relfilename, $mtime) = @_;
-
+	
 	defined($relfilename)||confess;
 	defined($mtime)||confess;
 	$partsize||confess;
-
+	
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/multipart-uploads";
 	$self->{method} = 'POST';
 
 	$self->add_header('x-amz-part-size', $partsize);
-
+	
 	# currently meat_encode only returns undef if filename is too big
 	defined($self->{description} = App::MtAws::MetaData::meta_encode($relfilename, $mtime)) or
 		die exception 'file_name_too_big' =>
@@ -87,7 +86,7 @@ sub create_multipart_upload
 		"Limit is about 700 ASCII characters or 350 2-byte UTF-8 character.",
 		filename => $relfilename;
 	$self->add_header('x-amz-archive-description', $self->{description});
-
+	
 	my $resp = $self->perform_lwp();
 	return $resp ? $resp->header('x-amz-multipart-upload-id') : undef;
 }
@@ -95,14 +94,14 @@ sub create_multipart_upload
 sub upload_part
 {
 	my ($self, $uploadid, $dataref, $offset, $part_final_hash) = @_;
-
+	
 	$uploadid||confess;
 	($self->{dataref} = $dataref)||confess;
 	defined($offset)||confess;
 	($self->{part_final_hash} = $part_final_hash)||confess;
-
+	
 	$self->_calc_data_hash;
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/multipart-uploads/$uploadid";
 	$self->{method} = 'PUT';
 	$self->add_header('Content-Type', 'application/octet-stream');
@@ -111,7 +110,7 @@ sub upload_part
 	$self->add_header('x-amz-sha256-tree-hash', $self->{part_final_hash});
 	my ($start, $end) = ($offset, $offset+length(${$self->{dataref}})-1 );
 	$self->add_header('Content-Range', "bytes ${start}-${end}/*");
-
+	
 	my $resp = $self->perform_lwp();
 	return $resp ? 1 : undef;
 }
@@ -124,7 +123,7 @@ sub finish_multipart_upload
 	$uploadid||confess;
 	$size||confess;
 	$treehash||confess;
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/multipart-uploads/$uploadid";
 	$self->{method} = 'POST';
 	$self->add_header('x-amz-sha256-tree-hash', $treehash);
@@ -140,10 +139,10 @@ sub delete_archive
 	my ($self, $archive_id) = @_;
 
 	$archive_id||confess;
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/archives/$archive_id";
 	$self->{method} = 'DELETE';
-
+	
 	my $resp = $self->perform_lwp();
 	return $resp ? 1 : undef;
 }
@@ -152,15 +151,14 @@ sub delete_archive
 sub retrieve_archive
 {
 	my ($self, $archive_id) = @_;
-
+	
 	$archive_id||confess;
-
+   
 	$self->add_header('Content-Type', 'application/x-www-form-urlencoded; charset=utf-8');
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs";
 	$self->{method} = 'POST';
 
 	#  add "SNSTopic": "sometopic"
-	# no Test::Tabs
 	my $body = <<"END";
 {
   "Type": "archive-retrieval",
@@ -168,9 +166,8 @@ sub retrieve_archive
 }
 END
 
-	# use Test::Tabs
 	$self->{dataref} = \$body;
-
+	
 	my $resp = $self->perform_lwp();
 	return $resp ? $resp->header('x-amz-job-id') : undef;
 }
@@ -178,22 +175,21 @@ END
 sub retrieve_inventory
 {
 	my ($self) = @_;
-
+	
 	$self->add_header('Content-Type', 'application/x-www-form-urlencoded; charset=utf-8');
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs";
 	$self->{method} = 'POST';
 
 	#  add "SNSTopic": "sometopic"
-	# no Test::Tabs
 	my $body = <<"END";
 {
   "Type": "inventory-retrieval",
   "Format": "JSON"
 }
 END
-	# use Test::Tabs
-	$self->{dataref} = \$body;
 
+	$self->{dataref} = \$body;
+	
 	my $resp = $self->perform_lwp();
 	return $resp ? $resp->header('x-amz-job-id') : undef;
 }
@@ -201,12 +197,12 @@ END
 sub retrieval_fetch_job
 {
 	my ($self, $marker) = @_;
-
+	
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs";
 
 	$self->{params} = { completed => 'true' };
 	$self->{params}->{marker} = $marker if defined($marker);
-
+	
 	$self->{method} = 'GET';
 
 	my $resp = $self->perform_lwp();
@@ -217,105 +213,77 @@ sub retrieval_fetch_job
 # TODO: rename
 sub retrieval_download_job
 {
-	my ($self, $jobid, $relfilename, $tempfile, $size, $journal_treehash) = @_;
-
-	$journal_treehash||confess;
-	$jobid||confess;
-	defined($tempfile)||confess;
-	defined($relfilename)||confess;
-	$size or confess "no size";
-
-	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
-
-	$self->{expected_size} = $size;
-	$self->{writer} = App::MtAws::HttpFileWriter->new(tempfile => $tempfile);
-
-	$self->{method} = 'GET';
-
-	my $resp = $self->perform_lwp();
-	my $reported_th = $resp->header('x-amz-sha256-tree-hash') or confess;
-
-	$self->{writer}->treehash->calc_tree();
-	my $th = $self->{writer}->treehash->get_final_hash();
-
-	$reported_th eq $th or
-		die exception 'treehash_mismatch_full' =>
-		'TreeHash for received file %string filename% (full file) does not match. '.
-		'TreeHash reported by server: %reported%, Calculated TreeHash: %calculated%, TreeHash from Journal: %journal_treehash%',
-		calculated => $th, reported => $reported_th, journal_treehash => $journal_treehash, filename => $relfilename;
-
-	$reported_th eq $journal_treehash or
-		die exception 'treehash_mismatch_journal' =>
-		'TreeHash for received file %string filename% (full file) does not match TreeHash in journal. '.
-		'TreeHash reported by server: %reported%, Calculated TreeHash: %calculated%, TreeHash from Journal: %journal_treehash%',
-		calculated => $th, reported => $reported_th, journal_treehash => $journal_treehash, filename => $relfilename;
-
-	return $resp ? 1 : undef;
-}
-
-sub segment_download_job
-{
-	my ($self, $jobid, $tempfile, $filename, $position, $size) = @_;
+	my ($self, $jobid, $filename) = @_;
 
 	$jobid||confess;
-	defined($position) or confess "no position";
-	$size or confess "no size";
 	defined($filename)||confess;
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
-
-	$self->{expected_size} = $size;
-	$self->{writer} = App::MtAws::HttpSegmentWriter->new(tempfile => $tempfile, position => $position, filename => $filename);
-
+	$self->{content_file} = binaryfilename $filename; # TODO: use temp filename for transactional behaviour
 	$self->{method} = 'GET';
-	my $end_position = $position + $size - 1;
-	$self->add_header('Range', "bytes=$position-$end_position");
 
 	my $resp = $self->perform_lwp();
-	$resp && $resp->code == 206 or confess;
-
-	my $reported_th = $resp->header('x-amz-sha256-tree-hash') or confess;
-	$self->{writer}->treehash->calc_tree();
-	my $th = $self->{writer}->treehash->get_final_hash();
-
-	$reported_th eq $th or
-		die exception 'treehash_mismatch_segment' =>
-		'TreeHash for received segment of file %string filename% (position %position%, size %size%) does not match. '.
-		'TreeHash reported by server %reported%, Calculated TreeHash %calculated%',
-		calculated => $th, reported => $reported_th, filename => $filename, position => $position, size => $size;
-		# TODO: better report relative filename
-
-	my ($start, $end, $len) = $resp->header('Content-Range') =~ m!bytes\s+(\d+)\-(\d+)\/(\d+)!;
-
-	confess unless defined($start) && defined($end) && $len;
-	confess unless $end >= $start;
-	confess unless $position == $start;
-	confess unless $end_position == $end;
-
 	return $resp ? 1 : undef; # $resp->decoded_content is undefined here as content_file used
 }
 
+
 sub retrieval_download_to_memory
 {
 	my ($self, $jobid) = @_;
 
 	$jobid||confess;
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
 	$self->{method} = 'GET';
 
 	my $resp = $self->perform_lwp();
+	
+	if ($ENV{MTGLACIER_DEBUG_INVENTORY}) {
+		my $d_content = $resp->decoded_content;
+		my $content = $resp->content;
+		
+		open FF, ">_download_inventory_headers.log";
+		print FF $resp->request->dump;
+		print FF $resp->dump;
+		print FF "\n\nContent length:[".length($d_content)."]\n";
+		print FF "Content match decoded content:[".($d_content eq $content)."]\n";
+		close FF;
 
-	$resp or confess;
-	return $resp->content;
+		open FF, ">_download_inventory_body.log";
+		print FF $d_content;
+		close FF;
+	}
+
+	if ($resp) {
+		my $r = $resp->decoded_content; # decoded_content is NOT binary string because MIME type is not text/*
+		confess if is_wide_string($r);
+		return $r;
+	} else {
+		return undef;
+	}
 }
 
+# TODO: remove
+sub download_inventory
+{
+	my ($self, $jobid) = @_;
+
+	$jobid||confess;
+   
+	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
+	$self->{method} = 'GET';
+
+	my $resp = $self->perform_lwp();
+	return $resp ? $resp : undef; # $resp->decoded_content is undefined here as content_file used
+}
+
+
 sub create_vault
 {
 	my ($self, $vault_name) = @_;
 
 	confess unless defined($vault_name);
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$vault_name";
 	$self->{method} = 'PUT';
 
@@ -328,7 +296,7 @@ sub delete_vault
 	my ($self, $vault_name) = @_;
 
 	confess unless defined($vault_name);
-
+   
 	$self->{url} = "/$self->{account_id}/vaults/$vault_name";
 	$self->{method} = 'DELETE';
 
@@ -353,86 +321,66 @@ sub _calc_data_hash
 sub _sign
 {
 	my ($self) = @_;
-
+	
 	my $now = time();
-
-	$self->{last_request_time} = $now;  # we use same timestamp when writing to journal
-
-	my $date8601 = strftime("%Y%m%dT%H%M%SZ", gmtime($now));
+	
+	$self->{last_request_time} = $now;
+	
+	my $date8601 = strftime("%Y%m%dT%H%M%SZ", gmtime($now)); # TODO: use same timestamp when writing to journal
 	my $datestr = strftime("%Y%m%d", gmtime($now));
-
-
+	 
+	
 	$self->{req_headers} = [
 		{ name => 'x-amz-date', value => $date8601 },
 	];
-
-
+	
+	
 	# getting canonical URL
-
+	
 	my @all_headers = sort { $a->{name} cmp $b->{name} } (@{$self->{headers}}, @{$self->{req_headers}});
-
-
+	
+	
 	my $canonical_headers = join ("\n", map { lc($_->{name}).":".trim($_->{value}) } @all_headers);
 	my $signed_headers = join (';', map { lc($_->{name}) } @all_headers);
-
+	
 	my $bodyhash = $self->{data_sha256} ?
 		$self->{data_sha256} :
 		( $self->{dataref} ? sha256_hex(${$self->{dataref}}) : sha256_hex('') );
-
+	
 	$self->{params_s} = $self->{params} ? join ('&', map { "$_=$self->{params}->{$_}" } sort keys %{$self->{params}}) : ""; # TODO: proper URI encode
 	my $canonical_query_string = $self->{params_s};
-
+	
 	my $canonical_url = join("\n", $self->{method}, $self->{url}, $canonical_query_string, $canonical_headers, "", $signed_headers, $bodyhash);
 	my $canonical_url_hash = sha256_hex($canonical_url);
 
-
+	
 	# /getting canonical URL
-
+	
 	my $credentials = "$datestr/$self->{region}/$self->{service}/aws4_request";
 
 	my $string_to_sign = join("\n", "AWS4-HMAC-SHA256", $date8601, $credentials, $canonical_url_hash);
 
 	my ($kSigning, $kSigning_hex) = get_signature_key($self->{secret}, $datestr, $self->{region}, $self->{service});
 	my $signature = hmac_sha256_hex($string_to_sign, $kSigning);
-
-
-
+	
+	
+	
 	my $auth = "AWS4-HMAC-SHA256 Credential=$self->{key}/$credentials, SignedHeaders=$signed_headers, Signature=$signature";
 
 	push @{$self->{req_headers}}, { name => 'Authorization', value => $auth};
 }
 
 
-sub _max_retries { 100 }
-sub _sleep($) { sleep shift }
-
-sub throttle
-{
-	my ($i) = @_;
-	if ($i <= 5) {
-		_sleep 1;
-	} elsif ($i <= 10) {
-		_sleep 5;
-	} elsif ($i <= 20) {
-		_sleep 15;
-	} elsif ($i <= 50) {
-		_sleep 60
-	} else {
-		_sleep 180;
-	}
-}
-
 sub perform_lwp
 {
 	my ($self) = @_;
-
-	for my $i (1.._max_retries) {
-		undef $self->{last_retry_reason};
+	
+	for my $i (1..100) {
 		$self->_sign();
 
-		my $ua = LWP::UserAgent->new(timeout => $self->{timeout});
+		my $ua = LWP::UserAgent->new(timeout => 120);
 		$ua->protocols_allowed ( [ 'https' ] ) if $self->{protocol} eq 'https'; # Lets hard code this.
-		$ua->agent("mt-aws-glacier/${App::MtAws::VERSION}beta (http://mt-aws.com/) "); # use of App::MtAws::VERSION_MATURITY instead of 'beta' produce warning
+		$ua->agent("mt-aws-glacier/$App::MtAws::VERSION (http://mt-aws.com/) "); 
 		my $req = undef;
 		my $url = $self->{protocol} ."://$self->{host}$self->{url}";
 		$url = $self->{protocol} ."://$ENV{MTGLACIER_FAKE_HOST}$self->{url}" if $ENV{MTGLACIER_FAKE_HOST};
@@ -445,17 +393,17 @@ sub perform_lwp
 		}
 		$url .= "?$self->{params_s}" if $self->{params_s};
 		if ($self->{method} eq 'PUT') {
-			$req = HTTP::Request->new(PUT => $url, undef, $self->{dataref});
+			$req = HTTP::Request::Common::PUT( $url, Content=>$self->{dataref});
 		} elsif ($self->{method} eq 'POST') {
 			if ($self->{dataref}) {
-				$req = HTTP::Request->new(POST => $url, [Content_Type => 'form-data'], ${$self->{dataref}});
+				$req = HTTP::Request::Common::POST( $url, Content_Type => 'form-data', Content=>${$self->{dataref}});
 			} else {
-				$req = HTTP::Request->new(POST => $url );
+				$req = HTTP::Request::Common::POST( $url );
 			}
 		} elsif ($self->{method} eq 'DELETE') {
-			$req = HTTP::Request->new(DELETE => $url);
+			$req = HTTP::Request::Common::DELETE( $url);
 		} elsif ($self->{method} eq 'GET') {
-			$req = HTTP::Request->new(GET => $url);
+			$req = HTTP::Request::Common::GET( $url);
 		} else {
 			confess;
 		}
@@ -465,94 +413,47 @@ sub perform_lwp
 		my $resp = undef;
 
 		my $t0 = time();
-		if ($self->{content_file} && $self->{writer}) {
-			confess "content_file and writer at same time";
-		} elsif ($self->{content_file}) {
+		if ($self->{content_file}) {
 			$resp = $ua->request($req, $self->{content_file});
-		} elsif ($self->{writer}) {
-			my $size = undef;
-			$resp = $ua->request($req, sub {
-				unless (defined($size)) {
-					if ($_[1] && $_[1]->isa('HTTP::Response')) {
-						$size = $_[1]->content_length;
-						if (!$size || ($self->{expected_size} && $size != $self->{expected_size})) {
-							die exception
-								wrong_file_size_in_journal =>
-									'Wrong Content-Length received from server, probably wrong file size in Journal or wrong server';
-						}
-						$self->{writer}->reinit($size);
-					} else {
-						# we should "confess" here, but we cant, only exceptions propogated
-						die exception "unknow_error" => "Unknown error, probably LWP version is too old";
-					}
-				}
-				$self->{writer}->add_data($_[0]);
-				1;
-			});
 		} else {
 			$resp = $ua->request($req);
 		}
 		my $dt = time()-$t0;
 
-		if (($resp->code eq '500') && $resp->header('Client-Warning') && ($resp->header('Client-Warning') eq 'Internal response')) {
-			print "PID $$ HTTP connection problem (timeout?). Will retry ($dt seconds spent for request)\n";
-			$self->{last_retry_reason} = 'Internal response';
-			throttle($i);
-		} elsif ($resp->code =~ /^(500|408)$/) {
+		if ($resp->code =~ /^(500|408)$/) {
 			print "PID $$ HTTP ".$resp->code." This might be normal. Will retry ($dt seconds spent for request)\n";
-			$self->{last_retry_reason} = $resp->code;
 			throttle($i);
-		} elsif (defined($resp->header('X-Died')) && (get_exception($resp->header('X-Died')))) {
-			die $resp->header('X-Died'); # propogate our own exceptions
-		} elsif (defined($resp->header('X-Died')) && length($resp->header('X-Died'))) {
-			print "PID $$ HTTP connection problem. Will retry ($dt seconds spent for request)\n";
-			$self->{last_retry_reason} = 'X-Died';
+		} elsif (defined($resp->header('X-Died')) && length($resp->header('X-Died')) && $resp->header('X-Died') =~ /^read timeout/i) {
+			print "PID $$ HTTP Timeout. Will retry ($dt seconds spent for request)\n";
 			throttle($i);
 		} elsif ($resp->code =~ /^2\d\d$/) {
-			if ($self->{writer}) {
-				my ($c, $reason) = $self->{writer}->finish();
-				if ($c eq 'retry') {
-					print "PID $$ HTTP $reason. Will retry ($dt seconds spent for request)\n";
-					$self->{last_retry_reason} = $reason;
-					throttle($i);
-				} elsif ($c ne 'ok') {
-					confess;
-				} else {
-					return $resp;
-				}
-			} elsif (defined($resp->content_length) && $resp->content_length != length($resp->content)){
-				print "PID $$ HTTP Unexpected end of data. Will retry ($dt seconds spent for request)\n";
-				$self->{last_retry_reason}='Unexpected end of data';
-				throttle($i);
-			} else {
-				return $resp;
-			}
+			return $resp;
 		} else {
-			if ($resp->code =~ /^40[03]$/) {
-				if ($resp->content_type && $resp->content_type eq 'application/json') {
-					my $json = JSON::XS->new->allow_nonref;
-					my $scalar = eval { $json->decode( $resp->content ); }; # we assume content always in utf8
-					if (defined $scalar) {
-						my $code = $scalar->{code};
-						my $type = $scalar->{type};
-						my $message = $scalar->{message};
-						if ($code eq 'ThrottlingException') {
-							print "PID $$ ThrottlingException. Will retry ($dt seconds spent for request)\n";
-							$self->{last_retry_reason} = 'ThrottlingException';
-							throttle($i);
-							next;
-						}
-					}
-				}
-			}
 			print STDERR "Error:\n";
-			print STDERR dump_request_response($req, $resp);
-			die exception 'http_unexpected_reply' => 'Unexpected reply from remote server';
+			print STDERR $req->dump;
+			print STDERR $resp->dump;
+			print STDERR "\n";
+			return undef;
 		}
 	}
-	die exception 'too_many_tries' => "Request was not successful after "._max_retries." retries";
+	return undef;
 }
 
+sub throttle
+{
+	my ($i) = @_;
+	if ($i <= 5) {
+		sleep 1;
+	} elsif ($i <= 10) {
+		sleep 5;
+	} elsif ($i <= 20) {
+		sleep 15;
+	} elsif ($i <= 50) {
+		sleep 60
+	} else {
+		sleep 180;
+	}
+}
 
 
 sub get_signature_key
@@ -578,4 +479,3 @@ sub trim
 
 
 1;
-__END__
