@@ -20,26 +20,23 @@
 
 package App::MtAws::RetrievalDownloadJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
 use base qw/App::MtAws::Job/;
-use Carp;
-use App::MtAws::Utils;
+use File::stat;
 
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	$self->{archives}||die;
-	$self->{pending}={};
-	$self->{all_raised} = 0;
-	$self->{position} = 0;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    $self->{archives}||die;
+    $self->{pending}={};
+    $self->{all_raised} = 0;
+    $self->{position} = 0;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -52,9 +49,7 @@ sub get_task
 		if (scalar @{$self->{archives}}) {
 			my $archive = shift @{$self->{archives}};
 			my $task = App::MtAws::Task->new(id => $archive->{jobid}, action=>"retrieval_download_job", data => {
-				archive_id => $archive->{archive_id}, relfilename => $archive->{relfilename},
-				filename => $archive->{filename}, mtime => $archive->{mtime}, jobid => $archive->{jobid},
-				size => $archive->{size}, treehash => $archive->{treehash}
+				archive_id => $archive->{archive_id}, relfilename => $archive->{relfilename}, filename => $archive->{filename}, mtime => $archive->{mtime}, jobid => $archive->{jobid}
 			});
 			$self->{pending}->{$archive->{jobid}}=1;
 			$self->{all_raised} = 1 unless scalar @{$self->{archives}};
@@ -70,8 +65,8 @@ sub finish_task
 {
 	my ($self, $task) = @_;
 	my $mtime = $task->{data}{mtime};
-	utime $mtime, $mtime, binaryfilename($task->{data}{filename}) or confess if defined $mtime;
-
+	utime $mtime, $mtime, $task->{data}{filename} if defined $mtime; # TODO: is that good that one process writes file and another one change it's mtime?
+	
 	delete $self->{pending}->{$task->{id}};
 	if ($self->{all_raised} && scalar keys %{$self->{pending}} == 0) {
 		return ("done");
@@ -79,5 +74,5 @@ sub finish_task
 		return ("ok");
 	}
 }
-
-1;
+	
+1;
\ No newline at end of file
