@@ -20,30 +20,26 @@
 
 package App::MtAws::RetrievalFetchJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
 use base qw/App::MtAws::Job/;
 use App::MtAws::FileUploadJob;
-use App::MtAws::SingleDownloadJob;
-use App::MtAws::SegmentDownloadJob;
 use App::MtAws::RetrievalDownloadJob;
-use Carp;
+
 use JSON::XS;
 
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	$self->{archives}||die;
-	$self->{raised} = 0;
-	$self->{downloads} ||= [];
-	$self->{seen} ||= {};
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    $self->{archives}||die;
+    $self->{raised} = 0;
+    $self->{downloads} ||= [];
+    $self->{seen} ||= {};
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -79,20 +75,9 @@ sub finish_task
 		}
 		
 		if ($scalar->{Marker}) {
-			return ("ok replace", App::MtAws::RetrievalFetchJob->new(archives => $self->{archives}, file_downloads => $self->{file_downloads},
-			downloads => $self->{downloads}, seen => $self->{seen}, marker => $scalar->{Marker} ) ); # TODO: we don't need go pagination if we have all archives to download
+			return ("ok replace", App::MtAws::RetrievalFetchJob->new(archives => $self->{archives}, downloads => $self->{downloads}, seen => $self->{seen}, marker => $scalar->{Marker} ) ); # TODO: we don't need go pagination if we have all archives to download
 		} elsif (scalar @{$self->{downloads}}) {
-			 #TODO allow parallel downloads while fetching job list
-			if ($self->{file_downloads}{'segment-size'}) {
-				return ("ok replace", App::MtAws::JobListProxy->new(jobs => [map {
-					confess unless $_->{size};
-					$_->{size} > $self->{file_downloads}{'segment-size'}*1048576 ?
-						App::MtAws::SegmentDownloadJob->new(file_downloads => $self->{file_downloads}, archive=>$_) :
-						App::MtAws::SingleDownloadJob->new(file_downloads => $self->{file_downloads}, archive=>$_)
-				} @{$self->{downloads}}]));
-			} else {
-				return ("ok replace", App::MtAws::RetrievalDownloadJob->new(file_downloads => $self->{file_downloads}, archives=>$self->{downloads}));
-			}
+			return ("ok replace", App::MtAws::RetrievalDownloadJob->new(archives=>$self->{downloads})); #TODO allow parallel downloads while fetching job list
 		} else {
 			return ("done");
 		}
@@ -101,4 +86,4 @@ sub finish_task
 	}
 }
 	
-1;
+1;
\ No newline at end of file
