@@ -20,18 +20,17 @@
 
 package App::MtAws::Utils;
 
-our $VERSION = '0.981_01';
 
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 use utf8;
 use File::Spec;
-use File::stat;
 use Carp;
 use Encode;
+use POSIX;
 use App::MtAws::Exceptions;
-use LWP::UserAgent;
-use bytes ();
+use bytes;
+no bytes;
 
 require Exporter;
 use base qw/Exporter/;
@@ -39,7 +38,7 @@ use base qw/Exporter/;
 
 our @EXPORT = qw/set_filename_encoding get_filename_encoding binaryfilename
 sanity_relative_filename is_relative_filename open_file sysreadfull syswritefull hex_dump_string
-is_wide_string characterfilename try_drop_utf8_flag dump_request_response file_size file_mtime file_exists/;
+is_wide_string/;
 
 # Does not work with directory names
 sub sanity_relative_filename
@@ -54,15 +53,11 @@ sub sanity_relative_filename
 	return $filename;
 }
 
-sub is_relative_filename
+sub is_relative_filename # TODO: test
 {
 	my ($filename) = @_;
-	return unless (defined($filename) && length($filename));
-	return if $filename =~ tr{\r\n\t}{} or index($filename, '//') != -1 or substr($filename, 0, 1) eq '/';
-	return undef if $filename =~ m{
-		(^|/)\.\.?(/|$)
-	}x;
-	1;
+	my $newname = sanity_relative_filename($filename);
+	return defined($newname) && ($filename eq $newname); 
 }
 
 
@@ -73,13 +68,9 @@ sub get_filename_encoding() { $_filename_encoding || confess };
 
 sub binaryfilename(;$)
 {
-	encode(get_filename_encoding, @_ ? shift : $_, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
+	encode(get_filename_encoding, @_ ? shift : $_, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);	
 }
 
-sub characterfilename(;$)
-{
-	decode(get_filename_encoding, @_ ? shift : $_, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);
-}
 
 =pod
 
@@ -108,41 +99,41 @@ sub open_file($$%)
 	(undef, my $filename, my %args) = @_;
 	%args = (use_filename_encoding => 1, %args);
 	my $original_filename = $filename;
-
+	
 	my %checkargs = %args;
 	defined $checkargs{$_} && delete $checkargs{$_} for qw/use_filename_encoding mode file_encoding not_empty binary/;
 	confess "Unknown argument(s) to open_file: ".join(';', keys %checkargs) if %checkargs;
-
+	
 	confess 'Argument "mode" is required' unless defined($args{mode});
-	confess "unknown mode $args{mode}" unless $args{mode} =~ m!^\+?(<|>>?)$!;
+	confess "unknown mode $args{mode}" unless $args{mode} =~ m!^(<|>>?)$!;
 	my $mode = $args{mode};
-
+	
 	confess "not_empty can be used in read mode only"
 		if ($args{not_empty} && $args{mode} ne '<');
-
-
+		
+	
 	if (defined($args{file_encoding})) {
 		$mode .= ":encoding($args{file_encoding})";
 		confess "cannot use binary and file_encoding at same time'" if $args{binary};
 	} elsif (!$args{binary}) {
 		confess "there should be file encoding or 'binary'";
 	}
-
+	
 	if ($args{use_filename_encoding}) {
 		$filename = binaryfilename $filename;
 	}
-
+	
 	confess "File is not a plain file" if -e $filename && (! -f $filename);
 	confess "File should not be empty" if $args{not_empty} && (! -s $filename);
-
+	
 	open ($_[0], $mode, $filename) or return;
 	my $f = $_[0];
-
+	
 	confess unless -f $f; # check for race condition - it was a file when we last checked, but now it's a directory
 	confess if $args{not_empty} && (! -s $f);
-
+	
 	binmode $f if $args{binary};
-
+	
 	return $f;
 }
 
@@ -153,43 +144,15 @@ sub file_size($%)
 	if ($args{use_filename_encoding}) {
 		$filename = binaryfilename $filename;
 	}
-	confess "file not exists" unless -f $filename;
+	croak unless -f $filename;
 	return -s $filename;
 }
 
-sub file_exists($%)
-{
-	my $filename = shift;
-	my (%args) = (use_filename_encoding => 1, @_);
-	if ($args{use_filename_encoding}) {
-		$filename = binaryfilename $filename;
-	}
-	return -f $filename;
-}
-
-sub file_mtime($%)
-{
-	my $filename = shift;
-	my (%args) = (use_filename_encoding => 1, @_);
-	if ($args{use_filename_encoding}) {
-		$filename = binaryfilename $filename;
-	}
-	confess "file not exists" unless -f $filename;
-	return stat($filename)->mtime;
-}
-
 sub is_wide_string
 {
 	defined($_[0]) && utf8::is_utf8($_[0]) && (bytes::length($_[0]) != length($_[0]))
 }
 
-# if we have ASCII-only data, let's drop UTF-8 flag in order to optimize some regexp stuff
-# TODO: write also version which does not check is_utf8 - it's faster when utf8 always set
-sub try_drop_utf8_flag
-{
-	Encode::_utf8_off($_[0]) if utf8::is_utf8($_[0]) && (bytes::length($_[0]) == length($_[0]));
-}
-
 sub sysreadfull($$$)
 {
 	my ($file, $len) = ($_[0], $_[2]);
@@ -198,14 +161,14 @@ sub sysreadfull($$$)
 		my $i = sysread($file, $_[1], $len - $n, $n);
 		if (defined($i)) {
 			if ($i == 0) {
-				return $n;
+				return 0;
 			} else {
 				$n += $i;
 			}
-		} elsif ($!{EINTR}) {
+		} elsif ($! == EINTR) {
 			redo;
 		} else {
-			return $n ? $n : undef;
+			return undef;
 		}
 	}
 	return $n;
@@ -220,10 +183,10 @@ sub syswritefull($$)
 		my $i = syswrite($file, $_[1], $len - $n, $n);
 		if (defined($i)) {
 			$n += $i;
-		} elsif ($!{EINTR}) {
+		} elsif ($! == EINTR) {
 			redo;
 		} else {
-			return $n ? $n : undef;
+			return undef;
 		}
 	}
 	return $n;
@@ -245,35 +208,6 @@ sub hex_dump_string
 	$str;
 }
 
-sub dump_request_response
-{
-	my ($req, $resp) = @_;
-	my $out = '';
-	$out .= "===REQUEST:\n";
-	$out .= join(" ", $req->method, $req->uri)."\n";
-
-	my $req_headers = $req->headers->as_string;
-
-	$req_headers =~ s!^(Authorization:.*Credential=)([A-Za-z0-9]+)/!$1***REMOVED***/!;
-	$req_headers =~ s!^(Authorization:.*Signature=)([A-Za-z0-9]+)!$1***REMOVED***!;
-
-	$out .= $req_headers;
-
-	if ($req->content_type ne 'application/octet-stream' && $req->content && length($req->content)) {
-		$out .= "\n".$req->content;
-	}
-
-	$out .= "\n===RESPONSE:\n";
-	$out .= $resp->protocol." " if $resp->protocol;
-	$out .= $resp->status_line."\n";
-	$out .= $resp->headers->as_string;
-
-	if ($resp->content_type eq 'application/json' && $resp->content && length($resp->content)) {
-		$out .= "\n".$resp->content;
-	}
-	$out .= "\n\n";
-	$out;
-}
 
 1;
 
